@@ -1,6 +1,8 @@
 import gazu
 import os
 import fnmatch
+import json
+from collections import defaultdict
 
 from kabaret import flow
 from kabaret.subprocess_manager.flow import RunAction
@@ -93,6 +95,46 @@ class KitsuAPIWrapper(flow.Object):
             name
         )
     
+    def get_shots(self, task_status_filters=None):
+        if task_status_filters is not None:
+            shots = []
+            shot_is_valid = defaultdict(lambda: False)
+            target_tasks = defaultdict(list)
+            
+            for type_name, statutes in task_status_filters.items():
+                type_id = gazu.task.get_task_type_by_name(type_name)['id']
+                
+                for status_name in statutes:
+                    status_id = gazu.task.get_task_status_by_name(status_name)['id']
+                    target_tasks[type_id].append(status_id)
+            
+            for t in gazu.task.all_tasks_for_project(self.get_project_id()):
+                if t['task_type_id'] not in target_tasks:
+                    continue
+                
+                shot_id = t['entity_id']
+                has_valid_status = False
+
+                # Loop over statutes
+                for status_id in target_tasks[t['task_type_id']]:
+                    if status_id == t['task_status_id']:
+                        has_valid_status = True
+                        break
+                
+                if has_valid_status:
+                    shots.append(gazu.shot.get_shot(shot_id))
+        else:
+            shots = gazu.shot.all_shots_for_project(
+                self.get_project_id()
+            )
+        
+        shots = [
+            (gazu.shot.get_sequence_from_shot(s)['name'], s['name'])
+            for s in shots
+        ]
+        
+        return shots
+    
     def get_shot_casting(self, shot, sequence=None):
         if isinstance(shot, str):
             shot = self.get_shot_data(shot, sequence)
@@ -233,6 +275,15 @@ class KitsuAPIWrapper(flow.Object):
             if status is not None:
                 gazu.task.add_comment(task, status, comment=comment)
     
+    def upload_shot_preview(self, sequence_name, shot_name, task_type_name, task_status_name, file_path, comment="", user_name=None):
+        shot = self.get_shot_data(shot_name, sequence_name)
+        result = False
+        
+        if shot is not None:
+            result = self.upload_preview(shot, task_type_name, task_status_name, file_path, comment, user_name)
+        
+        return result
+    
     def upload_preview(self, kitsu_entity, task_type_name, task_status_name, file_path, comment="", user_name=None):
         # Get user
         user = self.get_user(user_name)
@@ -265,7 +316,14 @@ class KitsuAPIWrapper(flow.Object):
             return False
         
         comment = gazu.task.add_comment(task, task_status, comment=comment)
-        gazu.task.add_preview(task, comment, file_path)
+
+        try:
+            gazu.task.add_preview(task, comment, file_path)
+        except json.decoder.JSONDecodeError:
+            self.root().session().log_warning(
+                f'Invalid response from Gazu while uploading preview {file_path}'
+            )
+            pass
         
         return True
 
